#!/usr/bin/env python3
# @author yutent<yutent.io@gmail.com>
# @date 2023/08/08 14:07:26


import gi, os, json, shutil,hashlib

gi.require_version("WebKit2", "4.1")

from gi.repository import Gtk, Gdk, WebKit2, GLib, Gio
from gi.repository.GdkPixbuf import Pixbuf

from ._settings import Settings
from ._protocal import Protocal
from ._inject import Inject
from ._utils import get_monitor_info, pixbuf_to_dict, dict_to_pixbuf

__dir__ = os.path.dirname(os.path.realpath(__file__))


class WebviewEngine(WebKit2.WebView):
  def __init__(self, App):

    WebKit2.WebView.__init__(self)

    self.app = App
    self.clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
    self.display = Gdk.Display.get_default()

    settings = Settings(True, False)

    self.set_settings(settings)

    Inject(self).connect(self.call_by_js)

    Protocal(self, os.path.join(__dir__, '../webapp')).register()

    im = self.get_input_method_context()
    im.set_enable_preedit(True)

    # 解决输入法候选框跟随问题
    im.connect('preedit-started', self.on_preedit_changed)
    im.connect('preedit-changed', self.on_preedit_changed)
    im.connect('preedit-finished', self.on_preedit_changed)

    self.load_uri("app:///index.html")

  def on_preedit_changed(self, im):
    seat = self.display.get_default_seat()
    p = seat.get_pointer().get_position() # 光标位置
    x, y = self.get_position() # 窗口位置

    im.notify_focus_in()
    im.notify_cursor_area(p.x - x, p.y - y, 0, 0) # 修正输入法跟随


  def call_js(self, method, data = None):
    scripts = 'native.$emit("' + method + '",' + json.dumps(data) + ')'
    self.evaluate_javascript(scripts, -1)

  
  def call_by_js(self, webview, message):

    data = json.loads(message.get_js_value().to_json(0))

    event = data.get('event')
    callback = data.get('callback')
    params = data.get('data')
    output = None


    match event:
      case 'fs':
        filepath = params.get('filepath')

        if params['action'] == 'read':
          with open(filepath, params.get('mode')) as file:
            output = file.read()
            if params.get('mode').find('b') > -1:
              output = list(output)

        if params['action'] == 'write':
        
            # 调整以支持二进制数据写入
            with open(filepath, params.get('mode')) as file:
              buff = params['content']

              if params.get('mode').find('b') > -1:
                buff = bytes(buff)

              output = file.write(buff)

        if params['action'] == 'exists':
          output = os.path.exists(filepath)

        if params['action'] == 'list':
          with os.scandir(filepath) as entries:
            output = [{
              "name": it.name,
              "path": os.path.join(filepath, it.name),
              "is_dir": it.is_dir(),
              "size": it.stat().st_size,
              "atime": int(it.stat().st_atime),
              "mtime": int(it.stat().st_mtime),
            } for it in entries]

        if params['action'] == 'remove':
          if os.path.isfile(filepath):
            output = os.remove(filepath)
          elif os.path.isdir(filepath):
            output = os.removedirs(filename)

        if params['action'] == 'rename':
          if os.path.exists(filepath):
            output = shutil.move(filepath, params['target'])

        if params['action'] == 'copy':
          if os.path.exists(filepath):
            output = shutil.copy2(filepath, params['target'])

        if params['action'] == 'isfile':
          output = os.path.isfile(filepath)
   
        if params['action'] == 'isdir':
          output = os.path.isdir(filepath)


      case 'clipboard':
        # 读文本
        if params['action'] == 'wait_for_text':
          output = self.clipboard.wait_for_text()
        
        # 写文本
        elif params['action'] == 'set_text':
          self.clipboard.set_text(params['value'], -1)

        # 写图片
        elif params['action'] == 'set_image':
          image = params['value']
          # 前端传进来的值, 如果是路径的话, 直接读取
          if type(image) == str:
            image = Pixbuf.new_from_file(image)
          else:
            image = dict_to_pixbuf(image)

          self.clipboard.set_image(image)
          self.clipboard.store()

        # 读图片
        elif params['action'] == 'wait_for_image':
          output = self.clipboard.wait_for_image()
          output = pixbuf_to_dict(output, 'noname.png')

        # 清除剪切板
        elif params['action'] == 'clear':
          self.clipboard.clear()

      # 退出app
      case 'quit':
        self.app.close()

      # 读取图片, 返回图片像素数据
      case 'image':
        filename = params['value']
        pixbuf = Pixbuf.new_from_file(filename)
        output = pixbuf_to_dict(pixbuf, filename)

      case 'md5':
        output = hashlib.md5(str(params.get('value'))).hexdigest()

    # 有回调则返回结果
    if callback:
      self.call_js(callback, output)
      